@@ -20,6 +20,7 @@ import netaddr
 from neutron.plugins.common import utils as p_utils
 from neutron_lib.api.definitions import l3
 from neutron_lib.api.definitions import port_security as psec
+from neutron_lib.api.definitions import provider_net as pnet
 from neutron_lib import constants as const
 from neutron_lib import context as n_context
 from neutron_lib.plugins import directory
@@ -203,6 +204,9 @@ class OVNClient(object):
                            cidrs.strip())
 
     def create_port(self, port):
+        if utils.is_lsp_ignored(port):
+            return
+
         port_info = self._get_port_options(port)
         external_ids = {ovn_const.OVN_PORT_NAME_EXT_ID_KEY: port['name'],
                         ovn_const.OVN_DEVID_EXT_ID_KEY: port['device_id'],
@@ -274,6 +278,9 @@ class OVNClient(object):
                                 if_exists=False))
 
     def update_port(self, port, original_port, qos_options=None):
+        if utils.is_lsp_ignored(port):
+            return
+
         port_info = self._get_port_options(port, qos_options)
         external_ids = {ovn_const.OVN_PORT_NAME_EXT_ID_KEY: port['name'],
                         ovn_const.OVN_DEVID_EXT_ID_KEY: port['device_id'],
@@ -661,7 +668,8 @@ class OVNClient(object):
         update = {}
         router_name = utils.ovn_name(router_id)
         enabled = delta['router'].get('admin_state_up')
-        if enabled and enabled != original_router['admin_state_up']:
+        if enabled is not None and (
+                enabled != original_router['admin_state_up']):
             update['enabled'] = enabled
 
         # Check for change in name
@@ -698,6 +706,17 @@ class OVNClient(object):
         with self._nb_idl.transaction(check_error=True) as txn:
             txn.add(self._nb_idl.delete_lrouter(lrouter_name))
 
+    def get_candidates_for_scheduling(self, extnet):
+        if extnet.get(pnet.NETWORK_TYPE) in [const.TYPE_FLAT,
+                                             const.TYPE_VLAN]:
+            physnet = extnet.get(pnet.PHYSICAL_NETWORK)
+            if not physnet:
+                return []
+            chassis_physnets = self._sb_idl.get_chassis_and_physnets()
+            return [chassis for chassis, physnets in chassis_physnets.items()
+                    if physnet in physnets]
+        return []
+
     def create_router_port(self, router_id, port):
         """Create a logical router port."""
         lrouter = utils.ovn_name(router_id)
@@ -707,8 +726,12 @@ class OVNClient(object):
             'device_owner')
         columns = {}
         if is_gw_port:
+            context = n_context.get_admin_context()
+            candidates = self.get_candidates_for_scheduling(
+                self._plugin.get_network(context, port['network_id']))
             selected_chassis = self._ovn_scheduler.select(
-                self._nb_idl, self._sb_idl, lrouter_port_name)
+                self._nb_idl, self._sb_idl, lrouter_port_name,
+                candidates=candidates)
             columns['options'] = {
                 ovn_const.OVN_GATEWAY_CHASSIS_KEY: selected_chassis}
         with self._nb_idl.transaction(check_error=True) as txn:
@@ -781,14 +804,8 @@ class OVNClient(object):
                 tag = int(segid) if segid else None
                 self._create_provnet_port(txn, network, physnet, tag)
 
-        if config.is_ovn_metadata_enabled():
-            # Create a neutron port for DHCP/metadata services
-            port = {'port':
-                    {'network_id': network['id'],
-                     'tenant_id': '',
-                     'device_owner': const.DEVICE_OWNER_DHCP}}
-            p_utils.create_port(self._plugin, n_context.get_admin_context(),
-                                port)
+        self.create_metadata_port(n_context.get_admin_context(), network)
+
         return network
 
     def delete_network(self, network_id):
@@ -1037,6 +1054,9 @@ class OVNClient(object):
         self._process_security_group_rule(rule, is_add_acl=False)
 
     def _find_metadata_port(self, context, network_id):
+        if not config.is_ovn_metadata_enabled():
+            return
+
         ports = self._plugin.get_ports(context, filters=dict(
             network_id=[network_id], device_owner=['network:dhcp']))
         # There should be only one metadata port per network
@@ -1051,6 +1071,15 @@ class OVNClient(object):
                 if fixed_ip['subnet_id'] == subnet['id']:
                     return fixed_ip['ip_address']
 
+    def create_metadata_port(self, context, network):
+        if config.is_ovn_metadata_enabled():
+            # Create a neutron port for DHCP/metadata services
+            port = {'port':
+                    {'network_id': network['id'],
+                     'tenant_id': network['project_id'],
+                     'device_owner': const.DEVICE_OWNER_DHCP}}
+            p_utils.create_port(self._plugin, context, port)
+
     def update_metadata_port(self, context, network_id):
         """Update metadata port.
 
