@@ -52,6 +52,10 @@ class MetadataServiceReadyWaitTimeoutException(Exception):
     pass
 
 
+class OVNPortUpdateError(n_exc.BadRequest):
+    pass
+
+
 class OVNMechanismDriver(api.MechanismDriver):
     """OVN ML2 mechanism driver
 
@@ -375,6 +379,23 @@ class OVNMechanismDriver(api.MechanismDriver):
                 n_context.get_admin_context(), port_id, resources.PORT,
                 provisioning_blocks.DHCP_ENTITY)
 
+    def _validate_ignored_port(self, port, original_port):
+        if utils.is_lsp_ignored(port):
+            if not utils.is_lsp_ignored(original_port):
+                # From not ignored port to ignored port
+                msg = (_('Updating device_owner to %(device_owner)s for port '
+                         '%(port_id)s is not supported') %
+                       {'device_owner': port['device_owner'],
+                        'port_id': port['id']})
+                raise OVNPortUpdateError(resource='port', msg=msg)
+        elif utils.is_lsp_ignored(original_port):
+            # From ignored port to not ignored port
+            msg = (_('Updating device_owner for port %(port_id)s owned by '
+                     '%(device_owner)s is not supported') %
+                   {'port_id': port['id'],
+                    'device_owner': original_port['device_owner']})
+            raise OVNPortUpdateError(resource='port', msg=msg)
+
     def create_port_postcommit(self, context):
         """Create a port.
 
@@ -405,6 +426,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         state changes that it does not know or care about.
         """
         port = context.current
+        original_port = context.original
+        self._validate_ignored_port(port, original_port)
         utils.validate_and_get_data_from_binding_profile(port)
         if self._is_port_provisioning_required(port, context.host,
                                                context.original_host):
@@ -598,25 +621,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         # creation or when OVN reports that the port is down) must be removed.
         LOG.info("OVN reports status up for port: %s", port_id)
 
-        if config.is_ovn_metadata_enabled() and self._sb_ovn:
-            # Wait until metadata service has been setup for this port in the
-            # chassis it resides.
-            chassis, datapath = (
-                self._sb_ovn.get_logical_port_chassis_and_datapath(port_id))
-            try:
-                n_utils.wait_until_true(
-                    lambda: datapath in
-                    self._sb_ovn.get_chassis_metadata_networks(chassis),
-                    timeout=METADATA_READY_WAIT_TIMEOUT,
-                    exception=MetadataServiceReadyWaitTimeoutException)
-            except MetadataServiceReadyWaitTimeoutException:
-                # If we reach this point it means that metadata agent didn't
-                # provision the datapath for this port on its chassis. Either
-                # the agent is not running or it crashed. We'll complete the
-                # provisioning block though.
-                LOG.warning("Metadata service is not ready for port %s, check"
-                            " networking-ovn-metadata-agent status/logs.",
-                            port_id)
+        self._wait_for_metadata_provisioned_if_needed(port_id)
 
         # If this port is a subport, we need to update the host_id and set it
         # to its parent's. Otherwise, Neutron won't even try to bind it and
@@ -673,3 +678,30 @@ class OVNMechanismDriver(api.MechanismDriver):
         hosts = {host for host, phynets in host_phynets_map.items()
                  if phynet in phynets}
         segment_service_db.map_segment_to_hosts(context, segment.id, hosts)
+
+    def _wait_for_metadata_provisioned_if_needed(self, port_id):
+        """Wait for metadata service to be provisioned.
+
+        Wait until metadata service has been setup for this port in the chassis
+        it resides. If metadata is disabled, this function will return right
+        away.
+        """
+        if config.is_ovn_metadata_enabled() and self._sb_ovn:
+            # Wait until metadata service has been setup for this port in the
+            # chassis it resides.
+            chassis, datapath = (
+                self._sb_ovn.get_logical_port_chassis_and_datapath(port_id))
+            try:
+                n_utils.wait_until_true(
+                    lambda: datapath in
+                    self._sb_ovn.get_chassis_metadata_networks(chassis),
+                    timeout=METADATA_READY_WAIT_TIMEOUT,
+                    exception=MetadataServiceReadyWaitTimeoutException)
+            except MetadataServiceReadyWaitTimeoutException:
+                # If we reach this point it means that metadata agent didn't
+                # provision the datapath for this port on its chassis. Either
+                # the agent is not running or it crashed. We'll complete the
+                # provisioning block though.
+                LOG.warning("Metadata service is not ready for port %s, check"
+                            " networking-ovn-metadata-agent status/logs.",
+                            port_id)
