@@ -14,22 +14,64 @@ from neutron_lib import exceptions as n_exc
 from oslo_log import log
 import tenacity
 
-from neutron.agent.ovsdb import impl_idl
-from neutron.agent.ovsdb.native import idlutils
 from neutron_lib.utils import helpers
+from ovsdbapp.backend.ovs_idl import connection
+from ovsdbapp.backend.ovs_idl import idlutils
+from ovsdbapp.backend.ovs_idl import transaction as idl_trans
+from ovsdbapp.backend.ovs_idl import vlog
 
-from networking_ovn._i18n import _, _LI
+from networking_ovn._i18n import _
 from networking_ovn.common import config as cfg
 from networking_ovn.common import constants as ovn_const
 from networking_ovn.common import utils
 from networking_ovn.ovsdb import commands as cmd
 from networking_ovn.ovsdb import ovn_api
 from networking_ovn.ovsdb import ovsdb_monitor
-from networking_ovn.ovsdb import vlog
 
 
 LOG = log.getLogger(__name__)
 
+# NOTE(twilson) This block of code is to support both ovsdbapp 0.4.0 and 1.0
+# and can be removed after ovsdbapp 1.0 is released and becomes a requirement
+try:
+    from ovsdbapp.backend import ovs_idl
+    Backend = ovs_idl.Backend
+except AttributeError:
+    class Backend(object):
+        lookup_table = {}
+
+        def __init__(self, connection):
+            super(Backend, self).__init__()  # pylint: disable=bad-super-call
+            self.start_connection(connection)
+
+        @classmethod
+        def start_connection(cls, connection):
+            try:
+                if cls.ovsdb_connection is None:
+                    cls.ovsdb_connection = connection
+                    cls.ovsdb_connection.start()
+            except Exception as e:
+                connection_exception = OvsdbConnectionUnavailable(
+                    db_schema=cls.schema, error=e)
+                LOG.exception(connection_exception)
+                raise connection_exception
+
+        @property
+        def idl(self):
+            return self.__class__.ovsdb_connection.idl
+
+        @property
+        def tables(self):
+            return self.idl.tables
+
+        _tables = tables
+
+        def create_transaction(self, check_error=False, log_errors=True):
+            return idl_trans.Transaction(
+                self, self.__class__.ovsdb_connection,
+                self.__class__.ovsdb_connection.timeout,
+                check_error, log_errors)
+
 
 class OvsdbConnectionUnavailable(n_exc.ServiceUnavailable):
     message = _("OVS database connection to %(db_schema)s failed with error: "
@@ -44,64 +86,42 @@ def get_ovn_idls(driver, trigger):
     @tenacity.retry(
         wait=tenacity.wait_exponential(max=180),
         reraise=True)
-    def get_ovn_idl_retry(cls, driver, trigger):
-        LOG.info(_LI('Getting %(cls)s for %(trigger)s with retry'),
+    def get_ovn_idl_retry(cls):
+        LOG.info('Getting %(cls)s for %(trigger)s with retry',
                  {'cls': cls.__name__, 'trigger': trigger.im_class.__name__})
-        return cls(driver, trigger)
+        return cls(get_connection(cls, trigger, driver))
 
-    vlog.use_oslo_logger()
-    nb_ovn_idl = get_ovn_idl_retry(OvsdbNbOvnIdl, driver, trigger)
-    sb_ovn_idl = get_ovn_idl_retry(OvsdbSbOvnIdl, driver, trigger)
-    return nb_ovn_idl, sb_ovn_idl
+    vlog.use_python_logger(max_level=cfg.get_ovn_ovsdb_log_level())
+    return tuple(get_ovn_idl_retry(c) for c in (OvsdbNbOvnIdl, OvsdbSbOvnIdl))
 
 
-def get_connection(db_class, trigger=None):
-    # The trigger is the start() method of the NeutronWorker class
-    if trigger and trigger.im_class == ovsdb_monitor.OvnWorker:
-        cls = ovsdb_monitor.OvnConnection
-    else:
-        cls = ovsdb_monitor.OvnBaseConnection
-
+def get_connection(db_class, trigger=None, driver=None):
+    # The trigger is the start() method of the worker class
     if db_class == OvsdbNbOvnIdl:
-        return cls(cfg.get_ovn_nb_connection(),
-                   cfg.get_ovn_ovsdb_timeout(), 'OVN_Northbound')
+        args = (cfg.get_ovn_nb_connection(), 'OVN_Northbound')
+        cls = ovsdb_monitor.OvnNbIdl
     elif db_class == OvsdbSbOvnIdl:
-        return cls(cfg.get_ovn_sb_connection(),
-                   cfg.get_ovn_ovsdb_timeout(), 'OVN_Southbound')
+        args = (cfg.get_ovn_sb_connection(), 'OVN_Southbound')
+        cls = ovsdb_monitor.OvnSbIdl
+
+    if trigger and trigger.im_class == ovsdb_monitor.OvnWorker:
+        idl_ = cls.from_server(*args, driver=driver)
+    else:
+        if db_class == OvsdbSbOvnIdl:
+            idl_ = ovsdb_monitor.BaseOvnSbIdl.from_server(*args)
+        else:
+            idl_ = ovsdb_monitor.BaseOvnIdl.from_server(*args)
+    return connection.Connection(idl_, timeout=cfg.get_ovn_ovsdb_timeout())
 
 
-class OvsdbNbOvnIdl(ovn_api.API):
-
+class OvsdbNbOvnIdl(Backend, ovn_api.API):
+    schema = 'OVN_Northbound'
     ovsdb_connection = None
 
-    def __init__(self, driver, trigger=None):
-        super(OvsdbNbOvnIdl, self).__init__()
-        try:
-            if OvsdbNbOvnIdl.ovsdb_connection is None:
-                OvsdbNbOvnIdl.ovsdb_connection = get_connection(
-                    OvsdbNbOvnIdl, trigger)
-            if isinstance(OvsdbNbOvnIdl.ovsdb_connection,
-                          ovsdb_monitor.OvnConnection):
-                OvsdbNbOvnIdl.ovsdb_connection.start(driver)
-            else:
-                OvsdbNbOvnIdl.ovsdb_connection.start()
-            self.idl = OvsdbNbOvnIdl.ovsdb_connection.idl
-            self.ovsdb_timeout = cfg.get_ovn_ovsdb_timeout()
-        except Exception as e:
-            connection_exception = OvsdbConnectionUnavailable(
-                db_schema='OVN_Northbound', error=e)
-            LOG.exception(connection_exception)
-            raise connection_exception
-
-    @property
-    def _tables(self):
-        return self.idl.tables
-
-    def transaction(self, check_error=False, log_errors=True, **kwargs):
-        return impl_idl.Transaction(self,
-                                    OvsdbNbOvnIdl.ovsdb_connection,
-                                    self.ovsdb_timeout,
-                                    check_error, log_errors)
+    def __init__(self, connection):
+        super(OvsdbNbOvnIdl, self).__init__(connection)
+        self.idl._session.reconnect.set_probe_interval(
+            cfg.get_ovn_ovsdb_probe_interval())
 
     def create_lswitch(self, lswitch_name, may_exist=True, **columns):
         return cmd.AddLSwitchCommand(self, lswitch_name,
@@ -144,11 +164,17 @@ class OvsdbNbOvnIdl(ovn_api.API):
                     lswitch.external_ids):
                 continue
             ports = []
+            provnet_port = None
             for lport in getattr(lswitch, 'ports', []):
                 if ovn_const.OVN_PORT_NAME_EXT_ID_KEY in lport.external_ids:
                     ports.append(lport.name)
+                # Handle provider network port
+                elif lport.name.startswith(
+                        ovn_const.OVN_PROVNET_PORT_NAME_PREFIX):
+                    provnet_port = lport.name
             result.append({'name': lswitch.name,
-                           'ports': ports})
+                           'ports': ports,
+                           'provnet_port': provnet_port})
         return result
 
     def get_all_logical_routers_with_rports(self):
@@ -159,6 +185,8 @@ class OvsdbNbOvnIdl(ovn_api.API):
                  - 'static_routes': list of static routes dict.
                  - 'ports': dict of port_id in neutron (key) and networks on
                             port (value).
+                 - 'snats': list of snats dict
+                 - 'dnat_and_snats': list of dnat_and_snats dict
         """
         result = []
         for lrouter in self._tables['Logical_Router'].rows.values():
@@ -170,9 +198,26 @@ class OvsdbNbOvnIdl(ovn_api.API):
             sroutes = [{'destination': sroute.ip_prefix,
                         'nexthop': sroute.nexthop}
                        for sroute in getattr(lrouter, 'static_routes', [])]
+
+            dnat_and_snats = []
+            snat = []
+            for nat in getattr(lrouter, 'nat', []):
+                if nat.type == 'dnat_and_snat':
+                    dnat_and_snats.append({
+                        'logical_ip': nat.logical_ip,
+                        'external_ip': nat.external_ip,
+                        'type': nat.type})
+                elif nat.type == 'snat':
+                    snat.append({
+                        'logical_ip': nat.logical_ip,
+                        'external_ip': nat.external_ip,
+                        'type': nat.type})
+
             result.append({'name': lrouter.name.replace('neutron-', ''),
                            'static_routes': sroutes,
-                           'ports': lrports})
+                           'ports': lrports,
+                           'snats': snat,
+                           'dnat_and_snats': dnat_and_snats})
         return result
 
     def get_acls_for_lswitches(self, lswitch_names):
@@ -238,17 +283,17 @@ class OvsdbNbOvnIdl(ovn_api.API):
     def add_lrouter_port(self, name, lrouter, **columns):
         return cmd.AddLRouterPortCommand(self, name, lrouter, **columns)
 
-    def update_lrouter_port(self, name, lrouter, if_exists=True, **columns):
-        return cmd.UpdateLRouterPortCommand(self, name, lrouter,
-                                            if_exists, **columns)
+    def update_lrouter_port(self, name, if_exists=True, **columns):
+        return cmd.UpdateLRouterPortCommand(self, name, if_exists, **columns)
 
     def delete_lrouter_port(self, name, lrouter, if_exists=True):
         return cmd.DelLRouterPortCommand(self, name, lrouter,
                                          if_exists)
 
-    def set_lrouter_port_in_lswitch_port(self, lswitch_port, lrouter_port):
+    def set_lrouter_port_in_lswitch_port(self, lswitch_port, lrouter_port,
+                                         if_exists=True):
         return cmd.SetLRouterPortInLSwitchPortCommand(self, lswitch_port,
-                                                      lrouter_port)
+                                                      lrouter_port, if_exists)
 
     def add_acl(self, lswitch, lport, **columns):
         return cmd.AddACLCommand(self, lswitch, lport, **columns)
@@ -285,30 +330,31 @@ class OvsdbNbOvnIdl(ovn_api.API):
         return cmd.UpdateAddrSetExtIdsCommand(self, name, external_ids,
                                               if_exists)
 
-    def get_all_chassis_router_bindings(self, chassis_candidate_list=None):
+    def get_all_chassis_gateway_bindings(self,
+                                         chassis_candidate_list=None):
         chassis_bindings = {}
         for chassis_name in chassis_candidate_list or []:
             chassis_bindings.setdefault(chassis_name, [])
-        for lrouter in self._tables['Logical_Router'].rows.values():
-            if ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY not in (
-                    lrouter.external_ids):
+        for lrp in self._tables['Logical_Router_Port'].rows.values():
+            if not lrp.name.startswith('lrp-'):
                 continue
-            chassis_name = lrouter.options.get('chassis')
+            chassis_name = lrp.options.get(ovn_const.OVN_GATEWAY_CHASSIS_KEY)
             if not chassis_name:
                 continue
             if (not chassis_candidate_list or
                     chassis_name in chassis_candidate_list):
                 routers_hosted = chassis_bindings.setdefault(chassis_name, [])
-                routers_hosted.append(lrouter.name)
+                routers_hosted.append(lrp.name)
         return chassis_bindings
 
-    def get_router_chassis_binding(self, router_name):
+    def get_gateway_chassis_binding(self, gateway_name):
         try:
             router = idlutils.row_by_value(self.idl,
-                                           'Logical_Router',
+                                           'Logical_Router_Port',
                                            'name',
-                                           router_name)
-            chassis_name = router.options.get('chassis')
+                                           gateway_name)
+            chassis_name = router.options.get(
+                ovn_const.OVN_GATEWAY_CHASSIS_KEY)
             if chassis_name == ovn_const.OVN_GATEWAY_INVALID_CHASSIS:
                 return None
             else:
@@ -316,23 +362,26 @@ class OvsdbNbOvnIdl(ovn_api.API):
         except idlutils.RowNotFound:
             return None
 
-    def get_unhosted_routers(self, valid_chassis_list):
-        unhosted_routers = {}
-        for lrouter in self._tables['Logical_Router'].rows.values():
-            if ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY not in (
-                    lrouter.external_ids):
+    def get_unhosted_gateways(self, port_physnet_dict, chassis_physnets):
+        unhosted_gateways = {}
+        valid_chassis_list = list(chassis_physnets)
+        for lrp in self._tables['Logical_Router_Port'].rows.values():
+            if not lrp.name.startswith('lrp-'):
                 continue
-            chassis_name = lrouter.options.get('chassis')
+            chassis_name = lrp.options.get(ovn_const.OVN_GATEWAY_CHASSIS_KEY)
             if not chassis_name:
                 # Not a gateway router
                 continue
+            physnet = port_physnet_dict.get(lrp.name[len('lrp-'):])
             # TODO(azbiswas): Handle the case when a chassis is no
             # longer valid. This may involve moving conntrack states,
             # so it needs to discussed in the OVN community first.
             if (chassis_name == ovn_const.OVN_GATEWAY_INVALID_CHASSIS or
-                    chassis_name not in valid_chassis_list):
-                unhosted_routers[lrouter.name] = lrouter.options
-        return unhosted_routers
+                    chassis_name not in valid_chassis_list or
+                    (physnet and
+                     physnet not in chassis_physnets.get(chassis_name))):
+                unhosted_gateways[lrp.name] = lrp.options
+        return unhosted_gateways
 
     def add_dhcp_options(self, subnet_id, port_id=None, may_exists=True,
                          **columns):
@@ -352,6 +401,16 @@ class OvsdbNbOvnIdl(ovn_api.API):
                         'uuid': row.uuid}
         return None
 
+    def get_subnet_and_ports_dhcp_options(self, subnet_id):
+        ret_opts = []
+        for row in self._tables['DHCP_Options'].rows.values():
+            external_ids = getattr(row, 'external_ids', {})
+            if subnet_id == external_ids.get('subnet_id'):
+                ret_opts.append(
+                    {'cidr': row.cidr, 'options': dict(row.options),
+                     'external_ids': dict(external_ids), 'uuid': row.uuid})
+        return ret_opts
+
     def get_subnets_dhcp_options(self, subnet_ids):
         ret_opts = []
         for row in self._tables['DHCP_Options'].rows.values():
@@ -476,6 +535,14 @@ class OvsdbNbOvnIdl(ovn_api.API):
     def delete_nat_ip_from_lrport_peer_options(self, lport, nat_ip):
         return cmd.DeleteNatIpFromLRPortPeerOptionsCommand(self, lport, nat_ip)
 
+    def get_parent_port(self, lsp_name):
+        try:
+            lsp = idlutils.row_by_value(self.idl, 'Logical_Switch_Port',
+                                        'name', lsp_name)
+            return lsp.parent_name
+        except idlutils.RowNotFound:
+            return ''
+
     # Check for a column match in the table. If not found do a retry with
     # a stop delay of 10 secs. This function would be useful if the caller
     # wants to verify for the presence of a particular row in the table
@@ -495,36 +562,20 @@ class OvsdbNbOvnIdl(ovn_api.API):
             raise RuntimeError(msg)
 
 
-class OvsdbSbOvnIdl(ovn_api.SbAPI):
-
+class OvsdbSbOvnIdl(Backend, ovn_api.SbAPI):
+    schema = 'OVN_Southbound'
     ovsdb_connection = None
 
-    def __init__(self, driver, trigger=None):
-        super(OvsdbSbOvnIdl, self).__init__()
-        try:
-            if OvsdbSbOvnIdl.ovsdb_connection is None:
-                OvsdbSbOvnIdl.ovsdb_connection = get_connection(OvsdbSbOvnIdl,
-                                                                trigger)
-            if isinstance(OvsdbSbOvnIdl.ovsdb_connection,
-                          ovsdb_monitor.OvnConnection):
-                # We only need to know the content of Chassis in OVN_Southbound
-                OvsdbSbOvnIdl.ovsdb_connection.start(
-                    driver, table_name_list=['Chassis'])
-            else:
-                OvsdbSbOvnIdl.ovsdb_connection.start(
-                    table_name_list=['Chassis'])
-            self.idl = OvsdbSbOvnIdl.ovsdb_connection.idl
-            self.ovsdb_timeout = cfg.get_ovn_ovsdb_timeout()
-        except Exception as e:
-            connection_exception = OvsdbConnectionUnavailable(
-                db_schema='OVN_Southbound', error=e)
-            LOG.exception(connection_exception)
-            raise connection_exception
+    def __init__(self, connection):
+        super(OvsdbSbOvnIdl, self).__init__(connection)
+        self.idl._session.reconnect.set_probe_interval(
+            cfg.get_ovn_ovsdb_probe_interval())
 
     def _get_chassis_physnets(self, chassis):
         bridge_mappings = chassis.external_ids.get('ovn-bridge-mappings', '')
-        mapping_dict = helpers.parse_mappings(bridge_mappings.split(','))
-        return mapping_dict.keys()
+        mapping_dict = helpers.parse_mappings(bridge_mappings.split(','),
+                                              unique_values=False)
+        return list(mapping_dict.keys())
 
     def chassis_exists(self, hostname):
         try:
@@ -539,6 +590,12 @@ class OvsdbSbOvnIdl(ovn_api.SbAPI):
             chassis_info_dict[ch.hostname] = self._get_chassis_physnets(ch)
         return chassis_info_dict
 
+    def get_chassis_and_physnets(self):
+        chassis_info_dict = {}
+        for ch in self.idl.tables['Chassis'].rows.values():
+            chassis_info_dict[ch.name] = self._get_chassis_physnets(ch)
+        return chassis_info_dict
+
     def get_all_chassis(self, chassis_type=None):
         # TODO(azbiswas): Use chassis_type as input once the compute type
         # preference patch (as part of external ids) merges.
@@ -557,3 +614,52 @@ class OvsdbSbOvnIdl(ovn_api.SbAPI):
         return (chassis.external_ids.get('datapath-type', ''),
                 chassis.external_ids.get('iface-types', ''),
                 self._get_chassis_physnets(chassis))
+
+    def get_metadata_port_network(self, network):
+        for port in self.idl.tables['Port_Binding'].rows.values():
+            if str(port.datapath.uuid) == network and port.type == 'localport':
+                return port
+
+    def get_chassis_metadata_networks(self, chassis_name):
+        """Return a list with the metadata networks the chassis is hosting."""
+        try:
+            chassis = idlutils.row_by_value(self.idl, 'Chassis',
+                                            'name', chassis_name)
+        except idlutils.RowNotFound:
+            msg = _('Chassis %s does not exist') % chassis_name
+            raise RuntimeError(msg)
+        proxy_networks = chassis.external_ids.get(
+            'neutron-metadata-proxy-networks', None)
+        return proxy_networks.split(',') if proxy_networks else []
+
+    def set_chassis_metadata_networks(self, chassis, networks):
+        nets = ','.join(networks) if networks else ''
+        return cmd.UpdateChassisExtIdsCommand(
+            self, chassis, {'neutron-metadata-proxy-networks': nets},
+            if_exists=True)
+
+    def get_network_port_bindings_by_ip(self, network, ip_address):
+        port_list = []
+        for port in self.idl.tables['Port_Binding'].rows.values():
+            if (port.mac and str(port.datapath.uuid) == network and
+                    ip_address in port.mac[0].split(' ')):
+                port_list.append(port)
+        return port_list
+
+    def set_port_cidrs(self, name, cidrs):
+        return cmd.UpdatePortBindingExtIdsCommand(
+            self, name, {'neutron-port-cidrs': cidrs}, if_exists=True)
+
+    def get_ports_on_chassis(self, chassis):
+        ports = []
+        for port in self.idl.tables['Port_Binding'].rows.values():
+            if port.chassis and port.chassis[0].name == chassis:
+                ports.append(port)
+        return ports
+
+    def get_logical_port_chassis_and_datapath(self, name):
+        for port in self.idl.tables['Port_Binding'].rows.values():
+            if port.logical_port == name:
+                datapath = str(port.datapath.uuid)
+                chassis = port.chassis[0].name if port.chassis else None
+                return chassis, datapath
